<?php
/**
 * @package     VirtueMart.vmpayment
 * @subpackage  com_vrpayecommerce
 *
 * @copyright Copyright (c) 2004 - 2010 VirtueMart Team. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

// Import Joomla controller library
jimport('joomla.application.component.controller');

defined('DS') or define('DS', DIRECTORY_SEPARATOR);

if (!class_exists('VmConfig'))
{
	$path = JPATH_ROOT . DS . 'administrator' . DS . 'components' . DS . 'com_virtuemart';
	$path .= DS . 'helpers' . DS . 'config.php';

	if (file_exists($path))
	{
		include $path;
		VmConfig::loadConfig();
	}
	else
	{
		$application = JFactory::getApplication();
		$application->enqueueMessage(
			'VirtueMart Core is not installed, please install VirtueMart again,
			or uninstall the AIO component by the joomla extension manager'
		);

			return false;
	}
}

if (!class_exists('ShopFunctions'))
{
	include JPATH_VM_ADMINISTRATOR . DS . 'helpers' . DS . 'shopfunctions.php';
}

if (!class_exists('VRpayecommerceHelper'))
{
	$path = JPATH_SITE . DS . 'plugins' . DS . 'vmpayment' . DS . 'vrpayecommerce' . DS;
	$path .= 'vrpayecommerce' . DS . 'helpers' . DS . 'helper.php';

	include $path;
}

if (!class_exists('VRpayecommercePaymentCore'))
{
	$path = JPATH_SITE . DS . 'plugins' . DS . 'vmpayment' . DS . 'vrpayecommerce' . DS;
	$path .= 'vrpayecommerce' . DS . 'helpers' . DS . 'core.php';

	include $path;
}

/**
 * VrpayecommerceController class to support my payment information component
 *
 * @package     VirtueMart.vmpayment
 * @subpackage  com_vrpayecommerce
 * @since       1.0.0
 */
class VrpayecommerceController extends JControllerLegacy
{
	/**
	 * payment method name
	 *
	 * @var    string
	 * @since  1.0.0
	 */
	protected $paymentMethod = '';

	public function __construct()
	{
		parent::__construct();
		VmConfig::loadJLang('com_vrpayecommerce', true);
		$this->user = JFactory::getUser();

		if (!$this->user->id || $this->user->guest)
		{
			die('Restricted access');
		}
	}

	/**
	 * Set Default of registered payment data
	 *
	 * @param   string  $recurringId
	 * @param   string  $paymentGroup
	 * @return  void
	 * @since   1.0.0
	 */
	public function setRecurringDefault($recurringId, $paymentGroup)
	{
		$db = JFactory::getDBO();
		$db->setQuery(
			"UPDATE `#__virtuemart_payment_plg_vrpayecommerce_recurring`
			SET payment_default = '0' WHERE  payment_default = '1'
            AND payment_group = '" . $db->escape($paymentGroup) . "'"
		);
		$db->execute();
		$db->setQuery(
			"UPDATE `#__virtuemart_payment_plg_vrpayecommerce_recurring`
			SET payment_default = '1' WHERE  id = '" . (int) $recurringId .
			"' AND payment_group = '" . $db->escape($paymentGroup) . "'"
		);
		$db->execute();
	}

	/**
	 * Get currency
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getCurrency()
	{
		if (!class_exists('CurrencyDisplay'))
		{
			require JPATH_VM_ADMINISTRATOR . DS . 'helpers' . DS . 'currencydisplay.php';
		}

		$currency = CurrencyDisplay::getInstance();
		$currencyId = $currency->getId();

		return $currency->ensureUsingCurrencyCode($currencyId);
	}

	/**
	 * Get customer address
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getAddress()
	{
		$userModel = VmModel::getModel('user');
		$userModel->setId((int) $this->user->id);
		$addressListBT = $userModel->getUserAddressList($userModel->getId(), 'BT');
		$addressListST = $userModel->getUserAddressList($userModel->getId(), 'ST');
		$address = ((isset($addressListBT[0])) ? $addressListBT[0] : $addressListST[0]);

		return $address;
	}

	/**
	 * Get checkout ID to create payment widget
	 *
	 * @param   string  $paymentWidgetUrl
	 * @param   string  $recurringId
	 * @return  string
	 * @since   1.0.0
	 */
	public function getCheckoutIdRecurring(&$paymentWidgetUrl, $recurringId = false)
	{
		$address = $this->getAddress();
		$VrpayecommerceHelper = new VRpayecommerceHelper($this->paymentMethod);
		$transactionData = $VrpayecommerceHelper->getCredentials();

		$transactionData['customer']['email'] = $this->user->email;
		$transactionData['customer']['first_name'] = $address->first_name;
		$transactionData['customer']['last_name'] = $address->last_name;
		$transactionData['billing']['street'] = $address->address_1;
		$transactionData['billing']['zip'] = $address->zip;
		$transactionData['billing']['city'] = $address->city;
		$transactionData['billing']['country_code'] = shopFunctions::getCountryByID(
			$address->virtuemart_country_id,
			'country_2_code'
		);

		$transactionData['amount'] = $VrpayecommerceHelper->getRegisterAmount();
		$transactionData['currency'] = $this->getCurrency();
		$transactionData['customer_ip'] = $VrpayecommerceHelper->getCustomerIp();

		if ($this->paymentMethod == 'cc_saved')
		{
			$transactionData['3D']['amount'] = $transactionData['amount'];
			$transactionData['3D']['currency'] = $transactionData['currency'];
		}

		$transactionData['test_mode'] = $VrpayecommerceHelper->getTestMode();
		$transactionData['payment_type'] = $VrpayecommerceHelper->getPaymentType();
		$transactionData['payment_recurring'] = 'INITIAL';
		$transactionData['payment_registration'] = 'true';

		if ($recurringId)
		{
			$transactionData['transaction_id'] = $this->getReferenceId($recurringId);
		}
		else
		{
			$transactionData['transaction_id'] = $this->user->id;
		}

		$paymentWidgetUrl = VRpayecommercePaymentCore::getPaymentWidgetUrl($transactionData, $checkoutId);

		if (!$checkoutId)
		{
			$application = JFactory::getApplication();
			$application->redirect(JROUTE::_('index.php?option=com_vrpayecommerce&error=ERROR_GENERAL_REDIRECT'));
		}

		return $checkoutId;
	}

	/**
	 * Get layout
	 *
	 * @param   string  $layout
	 * @return  object
	 * @since   1.0.0
	 */
	public function getLayout($layout = 'default')
	{
		$document = JFactory::getDocument();
		$viewType = $document->getType();
		$viewName = JRequest::getCmd('view', 'vrpayecommerce');
		$viewLayout = JRequest::getCmd('layout', $layout);

		$view = $this->getView($viewName, $viewType, '', array('layout' => $viewLayout));
		$view->document = $document;

		return $view;
	}

	/**
	 * Displays my payment information page
	 *
	 * @param   boolean  $cachable
	 * @param   array  $urlparams
	 * @return  object
	 * @since   1.0.0
	 */
	public function display($cachable = false, $urlparams = array())
	{
		$setDefault = JRequest::getVar('setDefault');
		$recurringId = JRequest::getInt('id');
		$group = JRequest::getVar('payment_group');
		$VrpayecommerceHelper = new VRpayecommerceHelper($this->paymentMethod);

		if ($setDefault)
		{
			$this->setRecurringDefault($recurringId, $group);
		}

		$view = $this->getLayout();
		$action = JRequest::getVar('action');
		$error = JRequest::getVar('error');

		if ($error)
		{
			if ($error == 'ERROR_GENERAL_REDIRECT')
			{
				$view->error = JText::_($error);
			}
			elseif ($action == 'register')
			{
				$view->error = JText::_('ERROR_MC_ADD') . ' ' . JText::_($error);
			}
			elseif ($action == 'change')
			{
				$view->error = JText::_('ERROR_MC_UPDATE') . ' ' . JText::_($error);
			}
		}

		$view->isRecurringActive = $VrpayecommerceHelper->getConfig('general_recurring');
		$view->isCardsSavedActive = $VrpayecommerceHelper->getConfig('cc_saved_active');
		$view->isDDSavedActive = $VrpayecommerceHelper->getConfig('dd_saved_active');
		$view->isPayPalSavedActive = $VrpayecommerceHelper->getConfig('paypal_saved_active');

		if ($view->isRecurringActive)
		{
			if ($view->isCardsSavedActive)
			{
				$VrpayecommerceHelper = new VRpayecommerceHelper('cc_saved');
				$view->customerDataCC = $VrpayecommerceHelper->getRegistrations();
			}

			if ($view->isDDSavedActive)
			{
				$VrpayecommerceHelper = new VRpayecommerceHelper('dd_saved');
				$view->customerDataDD = $VrpayecommerceHelper->getRegistrations();
			}

			if ($view->isPayPalSavedActive)
			{
				$VrpayecommerceHelper = new VRpayecommerceHelper('paypal_saved');
				$view->customerDataPAYPAL = $VrpayecommerceHelper->getRegistrations();
			}

			$view->registerUrl = JROUTE::_('index.php?option=com_vrpayecommerce&task=registerpayment');
			$view->changeUrl = JROUTE::_('index.php?option=com_vrpayecommerce&task=changepayment');
			$view->deleteUrl = JROUTE::_('index.php?option=com_vrpayecommerce&task=deletepayment');
			$view->assetsPath = JURI::root(true) . '/plugins/vmpayment/vrpayecommerce/vrpayecommerce/assets/';
		}

		$view->display();

		return $this;
	}

	/**
	 * Displays form payment widget to register payment data
	 *
	 * @return  object
	 * @since   1.0.0
	 */
	public function registerPayment()
	{
		$view = $this->getLayout('register');

		$selectedPayment = JRequest::getVar('selected_payment');
		$this->paymentMethod = $selectedPayment;
		$VrpayecommerceHelper = new VRpayecommerceHelper($this->paymentMethod);

		$error = JRequest::getVar('error');

		if ($error)
		{
			$view->error = JText::_('ERROR_MC_ADD') . ' ' . JText::_($error);
		}

		$checkoutId = $this->getCheckoutIdRecurring($paymentWidgetUrl);

		$view->lang = $VrpayecommerceHelper->getLang();
		$view->testMode = $VrpayecommerceHelper->getTestMode();
		$view->brand = $VrpayecommerceHelper->getBrand();
		$view->redirect = $VrpayecommerceHelper->isRedirect();
		$view->paymentWidgetUrl = $paymentWidgetUrl;
		$view->cancelUrl = JROUTE::_('index.php?option=com_vrpayecommerce');
		$view->responseUrl = JROUTE::_(
			JURI::root() . 'index.php?option=com_vrpayecommerce&task=response&pm=' . $selectedPayment
		);
		$view->assetsPath = JURI::root(true) . '/plugins/vmpayment/vrpayecommerce/vrpayecommerce/assets/';

		$view->display();

		return $this;
	}

	/**
	 * Displays form payment widget to change payment data
	 *
	 * @return  object
	 * @since   1.0.0
	 */
	public function changePayment()
	{
		$view = $this->getLayout('change');

		$selectedPayment = JRequest::getVar('selected_payment');
		$recurringId = JRequest::getVar('id');
		$this->paymentMethod = $selectedPayment;
		$VrpayecommerceHelper = new VRpayecommerceHelper($this->paymentMethod);

		$error = JRequest::getVar('error');

		if ($error)
		{
			$view->error = JText::_('ERROR_MC_UPDATE') . ' ' . JText::_($error);
		}

		$checkoutId = $this->getCheckoutIdRecurring($paymentWidgetUrl, $recurringId);

		$view->lang = $VrpayecommerceHelper->getLang();
		$view->testMode = $VrpayecommerceHelper->getTestMode();
		$view->brand = $VrpayecommerceHelper->getBrand();
		$view->redirect = $VrpayecommerceHelper->isRedirect();
		$view->paymentWidgetUrl = $paymentWidgetUrl;
		$view->cancelUrl = JROUTE::_('index.php?option=com_vrpayecommerce');
		$view->responseUrl = JROUTE::_(
			JURI::root() . 'index.php?option=com_vrpayecommerce&task=response&pm=' .
			$selectedPayment . '&recurring_id=' . $recurringId
		);
		$view->assetsPath = JURI::root(true) . '/plugins/vmpayment/vrpayecommerce/vrpayecommerce/assets/';

		$view->display();

		return $this;
	}

	/**
	 * redirect to success page
	 *
	 * @param   string $identifier
	 * @since   1.0.0
	 */
	public function redirectSuccess($identifier)
	{
		$application = JFactory::getApplication();
		$application->redirect(JROUTE::_('index.php?option=com_vrpayecommerce'), JText::_($identifier));
	}

	/**
	 * Displays delete payment page
	 *
	 * @return  object
	 * @since   1.0.0
	 */
	public function deletePayment()
	{
		$view = $this->getLayout('delete');

		$id = JRequest::getVar('id');
		$selectedPayment = JRequest::getVar('selected_payment');
		$this->paymentMethod = $selectedPayment;

		$doDelete = JRequest::getVar('doDelete');

		if ($doDelete)
		{
			$response = $this->deleteRecurring($id);

			if ($response == 'ACK')
			{
				$this->redirectSuccess('SUCCESS_MC_DELETE');
			}
			else
			{
				$view->error = JText::_('ERROR_MC_DELETE');
			}
		}

		$view->id = $id;
		$view->selectedPayment = $selectedPayment;
		$view->cancelUrl = JROUTE::_('index.php?option=com_vrpayecommerce');
		$view->assetsPath = JURI::root(true) . '/plugins/vmpayment/vrpayecommerce/vrpayecommerce/assets/';

		$view->display();

		return $this;
	}

	/**
	 * Get reference id by id
	 *
	 * @param   string $recurringId
	 * @return  string
	 * @since   1.0.0
	 */
	public function getReferenceId($recurringId)
	{
		$db = JFactory::getDBO();
		$query = "SELECT ref_id FROM `#__virtuemart_payment_plg_vrpayecommerce_recurring` WHERE id = '";
		$db->setQuery($query . (int) $recurringId . "'");
		$referenceId = $db->loadAssoc();

		return $referenceId['ref_id'];
	}

	/**
	 * delete registered payment data
	 *
	 * @param   string $recurringId
	 * @return  array
	 * @since   1.0.0
	 */
	public function deleteRecurring($recurringId)
	{
		$VrpayecommerceHelper = new VRpayecommerceHelper($this->paymentMethod);
		$referenceId = $this->getReferenceId($recurringId);
		$transactionData = $VrpayecommerceHelper->getCredentials();
		$transactionData['transaction_id'] = $this->user->id;
		$transactionData['test_mode'] = $VrpayecommerceHelper->getTestMode();

		$deleteResponse = VRpayecommercePaymentCore::deleteRegisteredAccount($referenceId, $transactionData);

		$codeResult = $deleteResponse["result"]["code"];
		$errorIdentifier = VRpayecommercePaymentCore::getErrorIdentifier($codeResult);
		$transactionResult = VRpayecommercePaymentCore::getTransactionResult($codeResult);

		if ($transactionResult == "ACK")
		{
			$db = JFactory::getDBO();
			$query = "DELETE FROM `#__virtuemart_payment_plg_vrpayecommerce_recurring` WHERE id = '";
			$db->setQuery($query . (int) $recurringId . "'");
			$db->execute();
		}

		return $transactionResult;
	}

	/**
	 * redirect to error page
	 *
	 * @param   string $errorIdentifier
	 * @param   string $recurringId
	 * @return  object
	 * @since   1.0.0
	 */
	public function redirectErrorRecurring($errorIdentifier, $recurringId)
	{
		if ($recurringId)
		{
			$action = 'change';
			$recurringId  = '&id=' . $recurringId;
		}
		else
		{
			$action = 'register';
			$recurringId = '';
		}

		$application = JFactory::getApplication();
		$VrpayecommerceHelper = new VRpayecommerceHelper($this->paymentMethod);

		if ($VrpayecommerceHelper->isRedirect())
		{
			$application->redirect(
				JROUTE::_(
					'index.php?option=com_vrpayecommerce&action=' . $action . '&error=' . $errorIdentifier
				)
			);
		}
		else
		{
			$application->redirect(
				JROUTE::_(
					'index.php?option=com_vrpayecommerce&task=' . $action . '
					payment&selected_payment=' . $this->paymentMethod . $recurringId . '&error=' . $errorIdentifier
				)
			);
		}
	}

	/**
	 * process palpal recurring
	 *
	 * @param   string $recurringId
	 * @param   array $paymentStatus
	 * @param   array $transactionData
	 * @param   object $VrpayecommerceHelper
	 * @return  void
	 * @since   1.0.0
	 */
	public function processPayPalSavedRecurring($recurringId, $paymentStatus, $transactionData, $VrpayecommerceHelper)
	{
		$registrationId = $paymentStatus['id'];
		$transactionData['payment_type'] = 'DB';
		$response = VRpayecommercePaymentCore::useRegisteredAccount($registrationId, $transactionData);

		$codeResult = $response["result"]["code"];
		$errorIdentifier = VRpayecommercePaymentCore::getErrorIdentifier($codeResult);
		$dbResult = VRpayecommercePaymentCore::getTransactionResult($codeResult);

		if ($dbResult == "ACK")
		{
			$referenceId = $response['id'];
			$transactionData['payment_type'] = 'RF';
			VRpayecommercePaymentCore::backOfficeOperation($referenceId, $transactionData);
			$VrpayecommerceHelper->insertOrUpdateRegistration($recurringId, $registrationId, $paymentStatus);
		}
		elseif ($dbResult == "NOK")
		{
			$this->redirectErrorRecurring($errorIdentifier, $recurringId);
		}
		else
		{
			$this->redirectErrorRecurring('ERROR_UNKNOWN', $recurringId);
		}
	}

	/**
	 * process credit cards and Direct Debit recurring
	 *
	 * @param   string $recurringId
	 * @param   array $paymentStatus
	 * @param   array $transactionData
	 * @return  void
	 * @param   object $vrpayecommerceHelper
	 * @since   1.0.0
	 */
	public function processPaymentSavedRecurring($recurringId, $paymentStatus, $transactionData, $VrpayecommerceHelper)
	{
		$registrationId = $paymentStatus['registrationId'];
		$referenceId = $paymentStatus['id'];

		if ($VrpayecommerceHelper->isMultiChannel())
		{
			$transactionData['channel_id'] = $transactionData['channel_id_moto'];
		}

		if ($paymentStatus['paymentType'] == 'PA')
		{
			$transactionData['payment_type'] = "CP";
			$response = VRpayecommercePaymentCore::backOfficeOperation($referenceId, $transactionData);

			$codeResult = $response["result"]["code"];
			$errorIdentifier = VRpayecommercePaymentCore::getErrorIdentifier($codeResult);
			$transactionResult = VRpayecommercePaymentCore::getTransactionResult($codeResult);

			if ($transactionResult == "ACK")
			{
				$referenceId = $response['id'];
			}
			elseif ($transactionResult == "NOK")
			{
				$this->redirectErrorRecurring($errorIdentifier, $recurringId);
			}
			else
			{
				$this->redirectErrorRecurring('ERROR_UNKNOWN', $recurringId);
			}
		}

		$transactionData['payment_type'] = "RF";
		$response = VRpayecommercePaymentCore::backOfficeOperation($referenceId, $transactionData);

		$VrpayecommerceHelper->insertOrUpdateRegistration($recurringId, $registrationId, $paymentStatus);
	}

	/**
	 * process payment method recurring
	 *
	 * @param   string $recurringId
	 * @param   array $paymentStatus
	 * @param   array $transactionData
	 * @param   object $VrpayecommerceHelper
	 * @return  void
	 * @since   1.0.0
	 */
	public function processPaymentSuccessRecurring($recurringId, $paymentStatus, $transactionData, $VrpayecommerceHelper)
	{
		$transactionData['amount'] = $VrpayecommerceHelper->getRegisterAmount();
		$transactionData['currency'] = $this->getCurrency();

		if ($recurringId)
		{
			$transactionData['transaction_id'] = $paymentStatus['merchantTransactionId'];
		}
		else
		{
			$transactionData['transaction_id'] = $this->user->id;
		}

		$transactionData['test_mode'] = $VrpayecommerceHelper->getTestMode();
		$transactionData['payment_recurring'] = 'INITIAL';

		if ($this->paymentMethod == "paypal_saved")
		{
			$this->processPayPalSavedRecurring($recurringId, $paymentStatus, $transactionData, $VrpayecommerceHelper);
		}
		else
		{
			$this->processPaymentSavedRecurring($recurringId, $paymentStatus, $transactionData, $VrpayecommerceHelper);
		}

		if ($recurringId)
		{
			$referenceId = $paymentStatus['merchantTransactionId'];
			$response = VRpayecommercePaymentCore::deleteRegisteredAccount($referenceId, $transactionData);
			$this->redirectSuccess('SUCCESS_MC_UPDATE');
		}
		else
		{
			$this->redirectSuccess('SUCCESS_MC_ADD');
		}
	}

	/**
	 * Get response
	 *
	 * @return  void
	 * @since   1.0.0
	 */
	public function response()
	{
		$recurringId = JRequest::getVar('recurring_id');
		$pm = JRequest::getVar('pm');

		$this->paymentMethod = $pm;
		$VrpayecommerceHelper = new VRpayecommerceHelper($this->paymentMethod);

		if ($VrpayecommerceHelper->isVersionTracker())
		{
			$VrpayecommerceHelper->sendVersionTracker();
		}

		$checkoutId = JRequest::getVar('id');
		$transactionData = $VrpayecommerceHelper->getCredentials();

		$paymentStatus = VRpayecommercePaymentCore::getPaymentStatus($checkoutId, $transactionData);
		$codeResult = $paymentStatus["result"]["code"];
		$errorIdentifier = VRpayecommercePaymentCore::getErrorIdentifier($codeResult);
		$transactionResult = VRpayecommercePaymentCore::getTransactionResult($codeResult);

		if (!$paymentStatus)
		{
			$this->redirectErrorRecurring('ERROR_GENERAL_NORESPONSE', $recurringId);
		}
		else
		{
			if ($transactionResult == "ACK")
			{
				$this->processPaymentSuccessRecurring(
					$recurringId,
					$paymentStatus,
					$transactionData,
					$VrpayecommerceHelper
				);
			}
			elseif ($transactionResult == "NOK")
			{
				$this->redirectErrorRecurring($errorIdentifier, $recurringId);
			}
			else
			{
				$this->redirectErrorRecurring('ERROR_UNKNOWN', $recurringId);
			}
		}
	}
}
