<?php
/**
 * @package     VirtueMart.vmpayment
 * @subpackage  vrpayecommerce
 *
 * @copyright Copyright (c) 2004 - 2010 VirtueMart Team. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */

defined('_JEXEC') or die('Restricted access');

if (!class_exists('vmPSPlugin'))
{
	require JPATH_VM_PLUGINS . DS . 'vmpsplugin.php';
}

if (!class_exists('VRpayecommerceHelper'))
{
	require JPATH_SITE . DS . 'plugins' . DS . 'vmpayment' . DS . 'vrpayecommerce' . DS . 'vrpayecommerce' . DS . 'helpers' . DS . 'helper.php';
}

if (!class_exists('VRpayecommercePaymentCore'))
{
	require JPATH_SITE . DS . 'plugins' . DS . 'vmpayment' . DS . 'vrpayecommerce' . DS . 'vrpayecommerce' . DS . 'helpers' . DS . 'core.php';
}

/**
 * plgVmpaymentVrpayecommerce class to create payment method plugin in VirtueMart
 *
 * @package     VirtueMart.vmpayment
 * @subpackage  vrpayecommerce
 * @since       1.0.0
 */
class PlgVmpaymentVrpayecommerce extends vmPSPlugin
{
	/**
	 * plugin method list
	 *
	 * @var    array
	 * @since  1.0.0
	 */
	protected $pluginMethods = array(
		'cc_saved',
		'cc',
		'dd_saved',
		'dd',
		'giropay',
		'klarnainv',
		'klarnains',
		'paypal_saved',
		'paypal',
		'paydirekt',
		'sofort'
	);

	/**
	 * payment method
	 *
	 * @var    string
	 * @since  1.0.0
	 */
	protected $paymentMethod;

	/**
	 * plugin config
	 *
	 * @var    string
	 * @since  1.0.0
	 */
	protected $pluginConfig;

	/**
	 * Vr pay eCommerce helper class
	 *
	 * @var    object
	 * @since  1.0.0
	 */
	protected $VrpayecommerceHelper;

	public function __construct(& $subject, $config)
	{
		parent::__construct($subject, $config);
		$this->_loggable = true;
		$this->_tablepkey = 'id';
		$this->_tableId = 'id';
		$this->tableFields = array_keys($this->getTableSQLFields());
		$varsToPush = $this->getVarsToPush();
		$this->setConfigParameterable($this->_configTableFieldName, $varsToPush);
		$this->user = JFactory::getUser();
		$this->getVrpayecommerceTerm();
	}

	/**
	 * Get the parameters of a plugin
	 * This function extends from vmPSPlugin class in VirtueMart
	 *
	 * @return  array
	 * @since   1.0.0
	 */
	public function getVarsToPush()
	{
		$varsToPush = parent::getVarsToPush();
		$varsToPush['payment_logos'] = array('', 'char');
		$varsToPush['cost_per_transaction'] = array('', 'float');
		$varsToPush['cost_percent_total'] = array('', 'float');
		$varsToPush['tax_id'] = array(0, 'int');

		return $varsToPush;
	}

	/**
	 * Create the table for this plugin if it does not yet exist.
	 *
	 * @return  void
	 * @since   1.0.0
	 */
	protected function getVmPluginCreateTableSQL()
	{
		return $this->createTableSQL('Payment Vrpayecommerce Table');
	}

	/**
	 * Get VR pay eCommerce table fields
	 * This function extends from vmPlugin class in VirtueMart
	 *
	 * @return  array
	 * @since   1.0.0
	 */
	public function getTableSQLFields()
	{
		$SQLfields = array(
			'id' => 'int(11) UNSIGNED NOT NULL AUTO_INCREMENT',
			'virtuemart_order_id' => 'int(11) UNSIGNED',
			'order_number' => 'char(64)',
			'virtuemart_paymentmethod_id' => 'mediumint(1) UNSIGNED',
			'payment_name' => 'varchar(5000)',
			'payment_order_total' => 'decimal(15,5) NOT NULL',
			'payment_currency' => 'char(3)',
			'cost_per_transaction' => 'decimal(10,2)',
			'cost_percent_total' => 'decimal(10,2)',
			'tax_id' => 'smallint(1)',

			'transaction_id' => 'char(32)',
			'unique_id' => 'char(32)',
			'payment_method' => 'char(50)',
			'payment_type' => 'char(2)'
		);

		return $SQLfields;
	}

	/**
	 * Create payment recurring table
	 *
	 * @return  void
	 * @since   1.0.0
	 */
	private function createRecurringTable()
	{
		$recurringTableName = $this->_tablename . '_recurring';
		$query = "CREATE TABLE IF NOT EXISTS `" . $recurringTableName . "` (
					`id` int(11) UNSIGNED NOT NULL AUTO_INCREMENT,
					`cust_id` int(11) UNSIGNED,
					`payment_group` char(32),
					`brand` char(100),
					`holder` char(100),
					`email` char(100),
					`last4digits` char(4),
					`expiry_month` char(2),
					`expiry_year` char(4),
					`server_mode` char(4),
					`channel_id` char(32),
					`ref_id` char(32),
					`payment_default` boolean NOT NULL default '0',
					PRIMARY KEY (`id`)
			) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT='Recurring Vrpayecommerce Table' AUTO_INCREMENT=1 ;";

		$db = JFactory::getDBO();
		$db->setQuery($query);

		if (!$db->execute())
		{
			JError::raiseWarning(
				1,
				$recurringTableName . '::createRecurringTable: ' . JText::_('COM_VIRTUEMART_SQL_ERROR') . ' ' . $db->stderr(true)
			);
			echo $recurringTableName . '::createRecurringTable: ' . JText::_('COM_VIRTUEMART_SQL_ERROR') . ' ' . $db->stderr(true);
		}

		$query = "SHOW columns FROM `" . $recurringTableName . "` LIKE 'server_mode'";
		$db->setQuery($query);

		if (!$db->loadResult())
		{
			$query = "ALTER TABLE `" . $recurringTableName . "` ADD `server_mode` char(4) NOT NULL AFTER `expiry_year`, ADD `channel_id`
			char(32) NOT NULL AFTER `server_mode`";
			$db->setQuery($query);

			if (!$db->execute())
			{
				JError::raiseWarning(
					1,
					$recurringTableName . '::createRecurringTable: ' . JText::_('COM_VIRTUEMART_SQL_ERROR') . ' ' . $db->stderr(true)
				);
				echo $recurringTableName . '::createRecurringTable: ' . JText::_('COM_VIRTUEMART_SQL_ERROR') . ' ' . $db->stderr(true);
			}
		}
	}

	/**
	 * Add VR pay eCommerce order States
	 *
	 * @param   string  $code
	 * @param   string  $name
	 * @param   string  $ordering
	 * @return  void
	 * @since   1.0.0
	 */
	private function addOrderStates($code, $name, $ordering)
	{
		$db = JFactory::getDBO();
		$db->setQuery("SELECT * FROM `#__virtuemart_orderstates` WHERE order_status_code = '" . $code . "' AND order_status_name = '" . $name . "'");

		if (!$db->loadResult())
		{
			$db->setQuery("
				INSERT INTO `#__virtuemart_orderstates` (
				`virtuemart_orderstate_id`,
				`order_status_code`,
				`order_status_name`,
				`order_status_description`,
				`order_stock_handle`,
				`ordering`,
				`virtuemart_vendor_id`
				) VALUES (
				null,
				'" . $code . "',
				'" . $name . "',
				'',
				'R','" . $ordering . "',
				1)"
			);
		}

		$db->execute();
	}

	/**
	 * Create the table for this plugin if it does not yet exist.
	 * This functions checks if the called plugin is active one.
	 * When yes it is calling the standard method to create the tables
	 *
	 * @param   string  $jpluginId
	 * @return  void
	 * @since   1.0.0
	 */
	public function plgVmOnStoreInstallPaymentPluginTable($jpluginId)
	{
		$this->addOrderStates('I', 'In Review', 1);
		$this->addOrderStates('Z', 'Pre-Authorization of Payment', 2);
		$this->addOrderStates('A', 'Payment Accepted', 3);
		$this->createRecurringTable();

		return parent::onStoreInstallPluginTable($jpluginId);
	}

	/**
	 * This is for adding the input data of the payment method to the cart, after selecting
	 *
	 * @param   object  $cart
	 * @param   string  $msg
	 * @return  null if payment not selected; true if card infos are correct; string containing the errors id cc is not valid
	 * @since   1.0.0
	 */
	public function plgVmOnSelectCheckPayment(VirtueMartCart $cart, &$msg)
	{
		$session = JFactory::getSession();
		$vrpayecommerceSession = new stdClass;
		$vrpayecommercePaymentMethod = JRequest::getVar('vrpayecommerce_paymentmethod', '');
		$vrpayecommerceSession->virtuemart_payment_method_id = $cart->virtuemart_paymentmethod_id;
		$vrpayecommerceSession->vrpayecommerce_paymentmethod = $vrpayecommercePaymentMethod;
		$session->set('vrpayecommerce_pm', json_encode($vrpayecommerceSession), 'vm');

		return $this->OnSelectCheck($cart);
	}

	/**
	 * This event is fired to display the plugin methods in the cart (edit shipment/payment) for example
	 *
	 * @param   object  $cart
	 * @param   integer  $selected
	 * @param   string  $htmlIn
	 * @return  boolean True on success, false on failures, null when this plugin was not selected.
	 *         On errors, JError::raiseWarning (or JError::raiseError) must be used to set a message.
	 * @since   1.0.0
	 */
	public function plgVmDisplayListFEPayment(VirtueMartCart $cart, $selected = 0, &$htmlIn)
	{
		return $this->displayListFE($cart, $selected, $htmlIn);
	}

	/**
	 * Check if the payment conditions are fulfilled for this payment method
	 *
	 * @param   object  $cart
	 * @param   string  $method
	 * @param   string  $cart_prices
	 * @return  true: if the conditions are fulfilled, false otherwise.
	 * @since   1.0.0
	 */
	protected function checkConditions($cart, $method, $cart_prices)
	{
		return true;
	}

	/**
	 * Set payment method
	 *
	 * @param   string  $paymentMethod
	 * @return  null
	 * @since   1.0.0
	 */
	protected function setPaymentMethod($paymentMethod)
	{
		$this->paymentMethod = $paymentMethod;
	}

	/**
	 * Set Vr pay eCommerce helper
	 *
	 * @return  null
	 * @since   1.0.4
	 */
	protected function setVrpayecommerceHelper()
	{
		$this->VrpayecommerceHelper = new VRpayecommerceHelper($this->paymentMethod);
	}

	/**
	 * Set plugin config
	 *
	 * @param   string  $pluginConfig
	 * @return  null
	 * @since   1.0.0
	 */
	protected function setPluginConfig($pluginConfig)
	{
		$this->pluginConfig = $pluginConfig;
	}

	/**
	 * Get payment methods Logo except credit cards Logo
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	protected function getLogo()
	{
		$this->setVrpayecommerceHelper();

		switch ($this->paymentMethod)
		{
			case 'dd':
			case 'dd_saved':
				$logo = 'sepa.png';
				break;
			case 'paypal_saved':
				$logo = 'paypal.png';
				break;
			case 'klarnainv':
			case 'klarnains':
				if ($this->VrpayecommerceHelper->getLang() == 'de')
				{
					$logo = strtolower($this->paymentMethod) . '_de.png';
				}
				else
				{
					$logo = strtolower($this->paymentMethod) . '_en.png';
				}
				break;
			case 'sofort':
				if ($this->VrpayecommerceHelper->getLang() == 'de')
				{
					$logo = 'sofortuberweisung.png';
				}
				else
				{
					$logo = 'sofortbanking.gif';
				}
				break;
			default:
				$logo = strtolower($this->paymentMethod) . '.png';
				break;
		}

		return $logo;
	}

	/**
	 * Set and get credit cards Logo in HTML code
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	protected function getCardLogoHtml()
	{
		$cardLogoHtml = '';
		$logos = $this->getPluginConfig($this->paymentMethod . '_cards');

		if (is_array($logos))
		{
			foreach ($logos as $logo)
			{
				$cardLogoHtml .= '<img src="plugins/vmpayment/vrpayecommerce/vrpayecommerce/assets/images/' . strtolower($logo) . '.png"
				 style="margin:5px 0px 5px 5px; height:35px !important;"/>';
			}
		}

		return $cardLogoHtml;
	}

	/**
	 * Set and get payment methods Logo in HTML code
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	protected function getLogoHtml()
	{
		$logo = $this->getLogo();

		switch ($this->paymentMethod)
		{
			case 'cc':
			case 'cc_saved':
			case 'dc':
				$logoHtml = $this->getCardLogoHtml();
				break;
			default:
				$logoHtml = '<img src="plugins/vmpayment/vrpayecommerce/vrpayecommerce/assets/images/' . $logo . '"
				 style="margin:5px; height:35px !important;"/>';
				break;
		}

		return $logoHtml;
	}

	/**
	 * Get plugin config
	 *
	 * @param   string  $key
	 * @return  string
	 * @since   1.0.0
	 */
	protected function getPluginConfig($key)
	{
		return $this->pluginConfig->$key;
	}

	/**
	 * Displays plugin list when checkout payment
	 * This function extends from vmPSPlugin class in VirtueMart
	 *
	 * @param   string  $plugin
	 * @param   string  $selectedPlugin
	 * @param   string  $pluginSalesPrice
	 * @return  string
	 * @since   1.0.0
	 */
	protected function getPluginHtml($plugin, $selectedPlugin, $pluginSalesPrice)
	{
		$this->setPluginConfig($plugin);
		$session = JFactory::getSession();
		$vrpayecommerceSession = $session->get('vrpayecommerce_pm', 0, 'vm');
		$vrpayecommerceData['vrpayecommerce_paymentmethod'] = false;

		if ($vrpayecommerceSession)
		{
			$vrpayecommerceData = json_decode($vrpayecommerceSession, true);
		}

		$pluginMethodId = $this->_idName;
		$pluginName = $this->_psType . '_name';

		if (!class_exists('CurrencyDisplay'))
		{
			require JPATH_VM_ADMINISTRATOR . DS . 'helpers' . DS . 'currencydisplay.php';
		}

		$currency = CurrencyDisplay::getInstance();
		$costDisplay = "";

		if ($pluginSalesPrice)
		{
			$costDisplay = $currency->priceDisplay($pluginSalesPrice);
			$costDisplay = '<span class="' . $this->_type . '_cost"> (' . JText::_('COM_VIRTUEMART_PLUGIN_COST_DISPLAY') . $costDisplay . ")</span>";
		}

		$dynUpdate = '';

		if (VmConfig::get('oncheckout_ajax', false))
		{
			$dynUpdate = ' data-dynamic-update="1" ';
		}

		$checked = '';
		$html = '<div style="display:none">
		<input type="radio"' . $dynUpdate . ' name="' . $pluginMethodId . '" id="' . $this->_psType . '_id_' . $plugin->$pluginMethodId . '"   value="
		' . $plugin->$pluginMethodId . '" ' . $checked . ">"
			. '<label for="' . $this->_psType . '_id_' . $plugin->$pluginMethodId . '">' . '
			<span class="' . $this->_type . '">' . $plugin->$pluginName . $costDisplay . "</span></label></div>";

		if (!$plugin->general_recurring || $this->user->guest)
		{
			$removeMethods = array('cc_saved','dd_saved','paypal_saved');
		}

		if ($plugin->general_recurring && !$this->user->guest)
		{
			$removeMethods = array('cc','dd','paypal');
		}

		$pluginMethods = array_diff($this->pluginMethods, $removeMethods);

		foreach ($pluginMethods as $pm)
		{
			$sortedPluginMethods[$pm] = $this->getPluginConfig($pm . '_sort_order');
		}

		$keys   = array_keys($sortedPluginMethods);
		$values = array_values($sortedPluginMethods);
		array_multisort($values, $keys);
		$this->pluginMethods = $keys;

		foreach ($this->pluginMethods as $pm)
		{
			$this->setPaymentMethod($pm);
			$pmActive = $this->getPluginConfig($pm . '_active');

			if ($pmActive)
			{
				$checkedPaymentMethod = '';

				if ($selectedPlugin == $plugin->$pluginMethodId && $pm == $vrpayecommerceData['vrpayecommerce_paymentmethod'])
				{
					$checkedPaymentMethod = 'checked="checked"';
				}

				if ($this->getLogoHtml())
				{
					$html .= '<div><input onclick="document.getElementById(\'payment_id_' . $plugin->$pluginMethodId . '\').click();
					" type="radio" id="vrpayecommerce_paymentmethod" name="vrpayecommerce_paymentmethod" value="' . $pm . '" ' .
					$checkedPaymentMethod . '/>' . $this->getLogoHtml() . '</div>'
					. '<span class="' . $this->_type . '">' . $costDisplay . "</span><br />";
				}
			}
		}

		return $html;
	}

	/**
	 * Calculate the price (value, tax_id) of the selected method
	 * It is called by the calculator
	 * This function does NOT to be reimplemented. If not reimplemented, then the default values from this function are taken.
	 *
	 * @param   object  $cart
	 * @param   array  $cartPrices
	 * @param   string  $cartPricesName
	 * @return  boolean|null
	 * @since   1.0.0
	 */
	public function plgVmonSelectedCalculatePricePayment(VirtueMartCart $cart, array &$cartPrices, &$cartPricesName)
	{
		return $this->onSelectedCalculatePrice($cart, $cartPrices, $cartPricesName);
	}

	/**
	 * Checks how many plugins are available. If only one, the user will not have the choice. Enter edit_xxx page
	 * The plugin must check first if it is the correct type
	 *
	 * @param   object  $cart
	 * @param   array  $cartPrices
	 * @param   string  $paymentCounter
	 * @return  null if no plugin was found, 0 if more then one plugin was found,  virtuemart_xxx_id if only one plugin is found
	 * @since   1.0.0
	 */
	public function plgVmOnCheckAutomaticSelectedPayment(VirtueMartCart $cart, array $cartPrices = array(), &$paymentCounter)
	{
		return $this->onCheckAutomaticSelected($cart, $cartPrices, $paymentCounter);
	}

	/**
	 * Checks how many plugins are available. If only one, the user will not have the choice. Enter edit_xxx page
	 * The plugin must check first if it is the correct type
	 * This function extends from vmPSPlugin class in VirtueMart
	 *
	 * @param   object  $cart
	 * @param   array  $cartPrices
	 * @param   int  $methodCounter
	 * @return  null if no plugin was found, 0 if more then one plugin was found,  virtuemart_xxx_id if only one plugin is found
	 * @since   1.0.0
	 */
	public function onCheckAutomaticSelected(VirtueMartCart $cart, array $cartPrices = array(), &$methodCounter = 0)
	{
		$virtuemartPluginMethodId = 0;

		$nbMethod = $this->getSelectable($cart, $virtuemartPluginMethodId, $cartPrices);
		$methodCounter += $nbMethod;

		if ($nbMethod == null)
		{
			return null;
		}
		else
		{
			return 0;
		}
	}

	/**
	 * Get customer sex
	 *
	 * @param   string  $title
	 * @return  string
	 * @since   1.0.0
	 */
	protected function getCustomerSex($title)
	{
		switch ($title)
		{
			case 'Mr':
				$customerSex = 'M';
				break;
			case 'Mrs':
				$customerSex = 'F';
				break;
			default:
				$customerSex = '';
				break;
		}

		return $customerSex;
	}

	/**
	 * Get cart items
	 *
	 * @param   object $cart
	 * @return  array
	 * @since   1.0.0
	 */
	protected function getCartItems($cart)
	{
		$items = $cart->products;
		$pricesUnformatted = $cart->pricesUnformatted;
		$cartItems = array();
		$i = 0;

		foreach ($items as $key => $item)
		{
			foreach ($pricesUnformatted[$key]['DATax'] as $value)
			{
				$discount = ($value['1'] * 100) / $pricesUnformatted[$key]['basePriceWithTax'];
			}

			foreach ($pricesUnformatted[$key]['VatTax'] as $value)
			{
				$tax = $value['1'];
			}

			$cartItems[$i]['merchant_item_id'] = $item->virtuemart_product_id;
			$cartItems[$i]['discount'] = isset($discount) ? $discount : '';
			$cartItems[$i]['quantity'] = $item->quantity;
			$cartItems[$i]['name'] = $item->product_name;
			$cartItems[$i]['price'] = $pricesUnformatted[$key]['salesPrice'];
			$cartItems[$i]['tax'] = isset($tax) ? $tax : '';
			$i++;
		}

		return $cartItems;
	}

	/**
	 * Get parameters for Klarna
	 *
	 * @param   object $cart
	 * @param   object $address
	 * @return  array
	 * @since   1.0.2
	 */
	protected function getKlarnaParameters($cart, $address)
	{
		$klarnaParameters = array();

		if ($this->paymentMethod == 'klarnainv' || $this->paymentMethod == 'klarnains')
		{
			$klarnaParameters['customer']['sex'] = $this->getCustomerSex($address->title);
			$klarnaParameters['customer']['birthdate'] = $address->dob;
			$klarnaParameters['customer']['phone'] = $address->phone_1;
			$klarnaParameters['customParameters']['KLARNA_CART_ITEM1_FLAGS'] = 32;
			$klarnaParameters['cartItems'] = $this->getCartItems($cart);
		}

		if ($this->paymentMethod == 'klarnains')
		{
			$klarnaParameters['customParameters']['KLARNA_PCLASS_FLAG'] = $this->VrpayecommerceHelper->getConfig('klarnains_pclass');
		}

		return $klarnaParameters;
	}

	/**
	 * Get parameters for PayDirekt
	 *
	 * @param   object $order
	 * @param   object $address
	 * @return  array
	 * @since   1.0.2
	 */
	protected function getPaydirektParameters($order)
	{
		$paydirektParameters = array();

		if ($this->paymentMethod == 'paydirekt')
		{
			$paydirektParameters['customParameters']['PAYDIREKT_minimumAge'] = $this->VrpayecommerceHelper->getMinimumAge();
			$paydirektParameters['customParameters']['PAYDIREKT_payment.isPartial'] = $this->VrpayecommerceHelper->getPaymentIsPartial();
		}

		return $paydirektParameters;
	}

	/**
	 * Get parameters for registered payment
	 *
	 * @return  array
	 * @since   1.0.2
	 */
	protected function getRegistrationParameters()
	{
		$registrationParameters = array();

		if ($this->VrpayecommerceHelper->isRecurring())
		{
			$registrationParameters['payment_registration'] = 'true';

			$registrations = $this->VrpayecommerceHelper->getRegistrations();

			if ($registrations)
			{
				foreach ($registrations as $key => $value)
				{
					$registrationParameters['registrations'][$key] = $value['ref_id'];
				}
			}
		}

		return $registrationParameters;
	}

	/**
	 * Get checkout ID to create payment widget
	 *
	 * @param   object $cart
	 * @param   object $order
	 * @param   object $method
	 * @param   string $paymentWidgetUrl
	 * @return  array
	 * @since   1.0.2
	 */
	protected function getCheckoutId($cart, $order, $method, &$paymentWidgetUrl)
	{
		$address = ((isset($order['details']['BT'])) ? $order['details']['BT'] : $order['details']['ST']);
		$this->getPaymentCurrency($method);
		$paymentCurrency = shopFunctions::getCurrencyByID($method->payment_currency, 'currency_code_3');
		$totalInPaymentCurrency = vmPSPlugin::getAmountInCurrency($order['details']['BT']->order_total, $method->payment_currency);

		if ($totalInPaymentCurrency['value'] <= 0)
		{
			$this->redirectError('ERROR_GENERAL_REDIRECT');
		}

		$transactionData = $this->VrpayecommerceHelper->getCredentials();
		$transactionData['customer']['email'] = $address->email;
		$transactionData['customer']['first_name'] = $address->first_name;
		$transactionData['customer']['last_name'] = $address->last_name;
		$transactionData['billing']['street'] = $address->address_1;
		$transactionData['billing']['zip'] = $address->zip;
		$transactionData['billing']['city'] = $address->city;
		$transactionData['billing']['state'] = shopFunctions::getStateByID($address->virtuemart_state_id);
		$transactionData['billing']['country_code'] = shopFunctions::getCountryByID($address->virtuemart_country_id, 'country_2_code');
		$transactionData['amount'] = $totalInPaymentCurrency['value'];
		$transactionData['currency'] = $paymentCurrency;
		$transactionData['customer_ip'] = $this->VrpayecommerceHelper->getCustomerIp();

		if ($this->paymentMethod == 'cc_saved')
		{
			$transactionData['3D']['amount'] = $transactionData['amount'];
			$transactionData['3D']['currency'] = $transactionData['currency'];
		}

		$transactionData['test_mode'] = $this->VrpayecommerceHelper->getTestMode();
		$transactionData['payment_type'] = $this->VrpayecommerceHelper->getPaymentType();
		$transactionData['transaction_id'] = $order['details']['BT']->order_number;

		$transactionData = array_merge_recursive(
			$transactionData,
			$this->getKlarnaParameters($cart, $address),
			$this->getPaydirektParameters($order),
			$this->getRegistrationParameters()
		);

		if ($this->paymentMethod == 'paypal_saved')
		{
			unset($transactionData['payment_type']);
		}

		$this->setSessionTransactionData($transactionData);
		$paymentWidgetUrl = VRpayecommercePaymentCore::getPaymentWidgetUrl($transactionData, $checkoutId);

		return $checkoutId;
	}

	/**
	 * Displays payment widget view
	 *
	 * @param   object $cart
	 * @param   object $order
	 * @return  boolean|null
	 * @since   1.0.0
	 */
	public function plgVmConfirmedOrder($cart, $order)
	{
		if (!($method = $this->getVmPluginMethod($order['details']['BT']->virtuemart_paymentmethod_id)))
		{
			return null;
		}

		if (!$this->selectedThisElement($method->payment_element))
		{
			return false;
		}

		if (!class_exists('VirtueMartModelOrders'))
		{
			require JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php';
		}

		if (!class_exists('VirtueMartModelCurrency'))
		{
			require JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'currency.php';
		}

		$paymentMethod = JRequest::getVar('vrpayecommerce_paymentmethod', '');
		$this->setPluginConfig($method);
		$this->setPaymentMethod($paymentMethod);
		$this->setVrpayecommerceHelper();
		$checkoutId = $this->getCheckoutId($cart, $order, $method, $paymentWidgetUrl);

		if ($this->paymentMethod == 'paypal_saved')
		{
			$template = 'paypalsaved';
		}
		else
		{
			$template = 'displaypayment';
		}

		if ($checkoutId)
		{
			$html = $this->renderByLayout(
				$template,
				array(
						'lang' => $this->VrpayecommerceHelper->getLang(),
						'brand' => $this->VrpayecommerceHelper->getBrand(),
						'redirect' => $this->VrpayecommerceHelper->isRedirect(),
						'recurring' => $this->VrpayecommerceHelper->isRecurring(),
						'cancelUrl' => JROUTE::_('index.php?option=com_virtuemart&view=cart&Itemid=' . JRequest::getInt('Itemid')),
						'responseUrl' => JROUTE::_(
							JURI::root() .
							'index.php?option=com_virtuemart&view=pluginresponse&task=pluginresponsereceived&on=' .
							$order['details']['BT']->order_number .
							'&vpi=' . $order['details']['BT']->virtuemart_paymentmethod_id .
							'&pm=' . $this->paymentMethod . '&Itemid=' . JRequest::getInt('Itemid') . '&lang=' . JRequest::getCmd('lang', '')
						),
						'testMode' => $this->VrpayecommerceHelper->getTestMode(),
						'registrations' => $this->VrpayecommerceHelper->getRegistrations(),
						'paymentWidgetUrl' => $paymentWidgetUrl,
						'assetsPath' => JURI::root(true) . '/plugins/vmpayment/vrpayecommerce/vrpayecommerce/assets/'
					)
			);
		}
		else
		{
			$this->redirectError('ERROR_GENERAL_REDIRECT');
		}

		$cart->_confirmDone = false;
		$cart->_dataValidated = false;
		$cart->setCartIntoSession();
		JRequest::setVar('display_title', false);
		JRequest::setVar('html', $html);
	}

	/**
	 * insert order data into the database
	 *
	 * @param   object $order
	 * @param   string $paymentStatus
	 * @return  void
	 * @since   1.0.0
	 */
	public function storeVrpayecommerceInternalData($order, $paymentStatus)
	{
		if (VRpayecommercePaymentCore::isSuccessReview($paymentStatus['result']['code']))
		{
			$paymentType = 'IR';
		}
		else
		{
			$paymentType = $paymentStatus['paymentType'];
		}

		$dbValues['payment_name'] = $this->renderPluginName($this->pluginConfig);
		$dbValues['payment_method'] = $this->paymentMethod;
		$dbValues['order_number'] = $order['details']['BT']->order_number;
		$dbValues['virtuemart_paymentmethod_id'] = $order['details']['BT']->virtuemart_paymentmethod_id;
		$dbValues['payment_order_total'] = $paymentStatus['amount'];
		$dbValues['payment_currency'] = $paymentStatus['currency'];
		$dbValues['transaction_id'] = $paymentStatus['merchantTransactionId'];
		$dbValues['unique_id'] = $paymentStatus['id'];
		$dbValues['payment_type'] = $paymentType;

		$this->storePSPluginInternalData($dbValues);
	}

	/**
	 * process registered PayPal payment
	 *
	 * @param   string $registrationId
	 * @param   array $paymentStatus
	 * @param   string $method
	 * @param   object $order
	 * @return  boolean
	 * @since   1.0.2
	 */
	public function processRegisteredPayPalSaved($registrationId, &$paymentStatus, $method, $order)
	{
		$sessionTransactionData = $this->getSessionTransactionData();
		$sessionTransactionData['payment_type'] = 'DB';
		$sessionTransactionData['payment_recurring'] = 'REPEATED';
		$debitResult = VRpayecommercePaymentCore::useRegisteredAccount($registrationId, $sessionTransactionData);
		$codeResult = $debitResult["result"]["code"];
		$errorIdentifier = VRpayecommercePaymentCore::getErrorIdentifier($codeResult);
		$transactionResult = VRpayecommercePaymentCore::getTransactionResult($codeResult);

		if ($transactionResult == "ACK")
		{
			$paymentStatus = $debitResult;

			return true;
		}
		elseif ($transactionResult == "NOK")
		{
			$this->redirectError($errorIdentifier);

			return false;
		}
		else
		{
			$this->redirectError('ERROR_UNKNOWN');

			return false;
		}

		return false;
	}

	/**
	 * process PayPal payment registration
	 *
	 * @param   array $paymentStatus
	 * @param   string $method
	 * @param   object $order
	 * @return  boolean
	 * @since   1.0.4
	 */
	protected function processNewPayPalSaved(&$paymentStatus, $method, $order)
	{
		$registrationId = $paymentStatus['id'];
		$sessionTransactionData = $this->getSessionTransactionData();
		$sessionTransactionData['transaction_id'] = $paymentStatus['merchantTransactionId'];
		$sessionTransactionData['payment_recurring'] = 'INITIAL';
		$sessionTransactionData['test_mode'] = $this->VrpayecommerceHelper->getTestMode();
		$sessionTransactionData['payment_type'] = 'DB';

		$registrationResult = VRpayecommercePaymentCore::useRegisteredAccount($registrationId, $sessionTransactionData);
		$codeResult = $registrationResult["result"]["code"];
		$errorIdentifier = VRpayecommercePaymentCore::getErrorIdentifier($codeResult);
		$transactionResult = VRpayecommercePaymentCore::getTransactionResult($codeResult);

		if ($transactionResult == "ACK")
		{
			$this->VrpayecommerceHelper->insertPaymentAccount($registrationId, $paymentStatus, true);
			$paymentStatus = $registrationResult;

			return true;
		}
		elseif ($transactionResult == "NOK")
		{
			$this->redirectError($errorIdentifier);

			return false;
		}
		else
		{
			$this->redirectError('ERROR_UNKNOWN');

			return false;
		}

		return false;
	}

	/**
	 * process payment success
	 *
	 * @param   array $paymentStatus
	 * @param   string $method
	 * @param   object $order
	 * @return  boolean
	 * @since   1.0.4
	 */
	protected function processPaymentSuccess(&$paymentStatus, $method, $order)
	{
		if ($this->VrpayecommerceHelper->isRecurring())
		{
			if ($this->paymentMethod == 'paypal_saved')
			{
				return $this->processNewPayPalSaved($paymentStatus, $method, $order);
			}
			else
			{
				$registrationId = $paymentStatus['registrationId'];
				$this->VrpayecommerceHelper->insertPaymentAccount($registrationId, $paymentStatus, true);

				return true;
			}
		}

		return true;
	}

	/**
	 * process Payment Response Received
	 *
	 * @param   string $html
	 * @return boolean|null|string
	 * @since   1.0.0
	 */
	public function plgVmOnPaymentResponseReceived(&$html)
	{
		if (!class_exists('VirtueMartCart'))
		{
			require JPATH_VM_SITE . DS . 'helpers' . DS . 'cart.php';
		}

		if (!class_exists('shopFunctionsF'))
		{
			require JPATH_VM_SITE . DS . 'helpers' . DS . 'shopfunctionsf.php';
		}

		if (!class_exists('VirtueMartModelOrders'))
		{
			require JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php';
		}

		VmConfig::loadJLang('com_virtuemart_orders', true);
		$virtuemartPaymentMethodId = JRequest::getInt('vpi', 0);
		$orderNumber = JRequest::getString('on', 0);

		if (!($method = $this->getVmPluginMethod($virtuemartPaymentMethodId)))
		{
			return null;
		}

		if (!$this->selectedThisElement($method->payment_element))
		{
			return null;
		}

		if (!($virtuemartOrderId = VirtueMartModelOrders::getOrderIdByOrderNumber($orderNumber)))
		{
			return null;
		}

		VmConfig::loadJLang('com_virtuemart');
		$orderModel = VmModel::getModel('orders');
		$order = $orderModel->getOrder($virtuemartOrderId);

		$checkoutId = JRequest::getString('id', 0);
		$paymentMethod = JRequest::getString('pm', 0);
		$registrationId = JRequest::getString('registrationId', 0);

		$this->setPluginConfig($method);
		$this->setPaymentMethod($paymentMethod);
		$this->setVrpayecommerceHelper();

		if ($this->VrpayecommerceHelper->isVersionTracker())
		{
			$this->VrpayecommerceHelper->sendVersionTracker();
		}

		$success = false;

		if ($registrationId && $this->paymentMethod == 'paypal_saved')
		{
			$success = $this->processRegisteredPayPalSaved($registrationId, $paymentStatus, $method, $order);
		}
		else
		{
			$transactionData = $this->VrpayecommerceHelper->getCredentials();
			$paymentStatus = VRpayecommercePaymentCore::getPaymentStatus($checkoutId, $transactionData);

			if (!$paymentStatus)
			{
				$this->redirectError('ERROR_GENERAL_NORESPONSE');
			}
			else
			{
				$codeResult = $paymentStatus["result"]["code"];
				$errorIdentifier = VRpayecommercePaymentCore::getErrorIdentifier($codeResult);
				$transactionResult = VRpayecommercePaymentCore::getTransactionResult($codeResult);

				if ($transactionResult == "ACK")
				{
					$success = $this->processPaymentSuccess($paymentStatus, $method, $order);
				}
				elseif ($transactionResult == 'NOK')
				{
					$this->redirectError($errorIdentifier);
				}
				else
				{
					$this->redirectError('ERROR_UNKNOWN');
				}
			}
		}

		if ($success)
		{
			$html = $this->doResultSuccess($virtuemartOrderId, $orderModel, $order, $paymentStatus);

			return true;
		}
	}

	/**
	 * process Payment Response Received
	 *
	 * @param   string $virtuemartOrderId
	 * @param   object $orderModel
	 * @param   object $order
	 * @param   array $paymentStatus
	 * @return string
	 * @since   1.0.2
	 */
	protected function doResultSuccess($virtuemartOrderId, $orderModel, $order, $paymentStatus)
	{
		$orderUpdate['customer_notified'] = 1;
		$orderUpdate['comments'] = '';
		$orderUpdate['order_status'] = $this->getStatus($paymentStatus);

		if ($orderUpdate['order_status'])
		{
			$orderModel->useDefaultEmailOrderStatus = false;
			$orderModel->updateStatusForOneOrder($virtuemartOrderId, $orderUpdate, true);
		}

		$this->storeVrpayecommerceInternalData($order, $paymentStatus);
		$html = $this->_getPaymentResponseHtml($order);
		$cart = VirtueMartCart::getCart();
		$cart->emptyCart();

		return $html;
	}

	/**
	 * process status payment
	 *
	 * @param   array $paymentStatus
	 * @return string|boolean
	 * @since   1.0.0
	 */
	protected function getStatus($paymentStatus)
	{
		if (VRpayecommercePaymentCore::isSuccessReview($paymentStatus['result']['code']))
		{
			return 'I';
		}
		elseif ($paymentStatus['paymentType'] == 'PA')
		{
			return 'Z';
		}
		elseif ($paymentStatus['paymentType'] == 'DB')
		{
			return 'A';
		}
		else
		{
			return false;
		}
	}

	/**
	 * Get payment response in HTML code
	 *
	 * @param   object $order
	 * @return string
	 * @since   1.0.0
	 */
	public function _getPaymentResponseHtml($order)
	{
		VmConfig::loadJLang('com_virtuemart');

		$html = '<table>' . "\n";
		$html .= $this->getHtmlRow('COM_VIRTUEMART_PAYMENT_NAME', $this->renderPluginName($this->pluginConfig));
		$html .= $this->getHtmlRow('COM_VIRTUEMART_PAYMENTMETHOD', JText::_('FRONTEND_PM_' . strtoupper($this->paymentMethod)));

		if (!empty($order['details']['BT']->order_number))
		{
			$html .= $this->getHtmlRow('COM_VIRTUEMART_ORDER_NUMBER', $order['details']['BT']->order_number);
		}

		$html .= '</table>' . "\n";

		$link = JRoute::_(
			"index.php?option=com_virtuemart&view=orders&layout=details
			&order_number=" . $order['details']['BT']->order_number . "&order_pass=" . $order['details']['BT']->order_pass,
			false
		);

		$html .= '<br />
		<a class="vm-button-correct" href="' . $link . '">' . JText::_('COM_VIRTUEMART_ORDER_VIEW_ORDER') . '</a>';

		return $html;
	}

	/**
	 * Displays VR pay eCommerce term message after installing plugin
	 *
	 * @return void
	 * @since  1.1.02
	 */
	public function getVrpayecommerceTerm()
	{
		if (!$this->isVrpayecommerceTermConfig())
		{
			$application = JFactory::getApplication();
			$application->enqueueMessage(
				JText::_('VRPAYECOMMERCE_TT_VERSIONTRACKER'), 'Warning'
			);
			$this->addVrpayecommerceTermConfig();
		}
	}

	/**
	 * Check availability VR pay eCommerce Term config into the database
	 *
	 * @return boolean
	 * @since  1.1.02
	 */
	public function isVrpayecommerceTermConfig()
	{
		$db = JFactory::getDBO();
		$query = "SELECT * FROM `#__messages_cfg` WHERE  cfg_name = 'vrpayecommerceTerm'";
		$db->setQuery($query);

		if (!$db->loadResult())
		{
			return false;
		}

		return true;
	}

	/**
	 * Insert VR pay eCommerce Term config into the database
	 *
	 * @return void
	 * @since  1.1.02
	 */
	public function addVrpayecommerceTermConfig()
	{
		$db = JFactory::getDBO();
		$db->setQuery("INSERT INTO `#__messages_cfg`
			(user_id, cfg_name, cfg_value)
			VALUES ('" . (int) $this->user->id . "', 'vrpayecommerceTerm', 'true')"
		);

		$db->execute();
	}

	/**
	 * redirect to error page
	 *
	 * @param   string $errorIdentifier
	 * @return  void
	 * @since   1.0.0
	 */
	protected function redirectError($errorIdentifier)
	{
		$app = JFactory::getApplication();
		$app->redirect(
			JRoute::_('index.php?option=com_virtuemart&view=cart&Itemid=' . JRequest::getInt('Itemid'), false),
			JText::_($errorIdentifier)
		);
	}

	/**
	 * Get transaction data
	 *
	 * @return  array
	 * @since   1.0.0
	 */
	protected function getSessionTransactionData()
	{
		$sessiontransactionData = array();
		$session = JFactory::getSession();
		$vrpayecommerceSession = $session->get('vrpayecommerce_transactionData', 0, 'vm');

		if ($vrpayecommerceSession)
		{
			$vrpayecommerceSession = json_decode($vrpayecommerceSession, true);
			$sessiontransactionData = $vrpayecommerceSession['transactionData'];
		}

		return $sessiontransactionData;
	}

	/**
	 * set transaction data
	 *
	 * @param   array $transactionData
	 * @return  void
	 * @since   1.0.0
	 */
	protected function setSessionTransactionData($transactionData)
	{
		$session = JFactory::getSession();
		$vrpayecommerceSession = new stdClass;
		$vrpayecommerceSession->transactionData = $transactionData;
		$session->set('vrpayecommerce_transactionData', json_encode($vrpayecommerceSession), 'vm');
	}

	/**
	 * Display stored payment data for an order
	 *
	 * @param   string $virtuemartOrderId
	 * @param   string $virtuemartPaymentId
	 * @return  string
	 * @since   1.0.0
	 */
	public function plgVmOnShowOrderBEPayment($virtuemartOrderId, $virtuemartPaymentId)
	{
		if (!$this->selectedThisByMethodId($virtuemartPaymentId))
		{
			return null;
		}

		if (!($paymentTable = $this->getDataByOrderId($virtuemartOrderId)))
		{
			return null;
		}

		VmConfig::loadJLang('com_virtuemart');
		$html = '<table class="adminlist table">' . "\n";
		$html .= $this->getHtmlHeaderBE();
		$html .= $this->getHtmlRowBE('COM_VIRTUEMART_PAYMENT_NAME', $paymentTable->payment_name);
		$html .= $this->getHtmlRowBE('COM_VIRTUEMART_PAYMENTMETHOD', JText::_('BACKEND_PM_' . strtoupper($paymentTable->payment_method)));
		$html .= $this->getHtmlRowBE('BACKEND_TT_TRANSACTION_ID', $paymentTable->transaction_id);

		$html .= $this->getHtmlRowBE('STANDARD_PAYMENT_TOTAL_CURRENCY', $paymentTable->payment_order_total . ' ' . $paymentTable->payment_currency);
		$html .= '</table>' . "\n";

		return $html;
	}

	/**
	 * Save updated order data to the method specific table
	 *
	 * @param   array $order
	 * @param   string $oldOrderStatus
	 * @return  boolean|null
	 * @since   1.0.0
	 */
	public function plgVmOnUpdateOrderPayment(&$order, $oldOrderStatus)
	{
		if (!($method = $this->getVmPluginMethod($order->virtuemart_paymentmethod_id)))
		{
			return null;
		}

		if (!$this->selectedThisElement($method->payment_element))
		{
			return null;
		}

		if (!($payments = $this->getDatasByOrderId($order->virtuemart_order_id)))
		{
			return null;
		}

		$payment = $payments[0];
		$this->setPluginConfig($method);
		$this->setPaymentMethod($payment->payment_method);
		$action = '';

		if ($oldOrderStatus == 'I' && $order->order_status == 'I')
		{
			$action = 'update';
			$result = $this->doUpdateStatus($order, $payment, $new_order_status);
			$order->order_status = $new_order_status;
		}
		elseif ($oldOrderStatus == 'Z' && $order->order_status == 'A')
		{
			$action = 'capture';
			$result = $this->doBackendTransaction($payment, 'CP');
		}
		elseif ($oldOrderStatus == 'A' && $order->order_status == 'R')
		{
			$action = 'refund';
			$result = $this->doBackendTransaction($payment, 'RF');
		}
		else
		{
			return null;
		}

		if (!$result)
		{
			vmError(JText::_('ERROR_' . strtoupper($action) . '_BACKEND'));

			return false;
		}

		return true;
	}

	/**
	 * update status payment
	 *
	 * @param   object $order
	 * @param   object $payment
	 * @param   string $newOrderStatus
	 * @return  boolean
	 * @since   1.0.0
	 */
	public function doUpdateStatus($order,$payment,&$newOrderStatus)
	{
		$this->setVrpayecommerceHelper();
		$referenceId = $payment->unique_id;
		$transactionData = $this->VrpayecommerceHelper->getCredentials();
		$transactionData['test_mode'] = $this->VrpayecommerceHelper->getTestMode();

		$result = VRpayecommercePaymentCore::updateStatus($referenceId, $transactionData);

		$xmlResult = simplexml_load_string($result);
		$returnCode = (string) $xmlResult->Result->Transaction->Processing->Return['code'];
		$transactionResult = VRpayecommercePaymentCore::getTransactionResult($returnCode);

		if ($transactionResult == 'ACK')
		{
			if (VRpayecommercePaymentCore::isSuccessReview($returnCode))
			{
				$newOrderStatus = 'I';
			}
			else
			{
				$paymentCode = (string) $xmlResult->Result->Transaction->Payment['code'];
				$paymentType = substr($paymentCode, -2);

				if ($paymentType == 'PA')
				{
					$newOrderStatus = 'Z';
				}

				if ($paymentType == 'DB')
				{
					$newOrderStatus = 'A';
				}

				$this->updateVmPlgPaymentType($payment, $paymentType);
			}

			return true;
		}

		return false;
	}

	/**
	 * do capture and refund payment
	 *
	 * @param   object $payment
	 * @param   string $paymentType
	 * @return  boolean
	 * @since   1.0.0
	 */
	protected function doBackendTransaction($payment,$paymentType)
	{
		$this->setVrpayecommerceHelper();
		$referenceId = $payment->unique_id;
		$transactionData = $this->VrpayecommerceHelper->getCredentials();

		if ($this->VrpayecommerceHelper->isMultiChannel())
		{
			$transactionData['channel_id'] = $transactionData['channel_id_moto'];
		}

		$transactionData['currency'] = $payment->payment_currency;
		$transactionData['amount'] = $payment->payment_order_total;
		$transactionData['test_mode'] = $this->VrpayecommerceHelper->getTestMode();
		$transactionData['payment_type'] = $paymentType;

		$response = VRpayecommercePaymentCore::backOfficeOperation($referenceId, $transactionData);

		$returnCode = $response["result"]["code"];
		$transactionResult = VRpayecommercePaymentCore::getTransactionResult($returnCode);

		if ($transactionResult == 'ACK')
		{
			$this->updateVmPlgPaymentType($payment, $paymentType);

			return true;
		}

		return false;
	}

	/**
	 * update payment type into the database
	 *
	 * @param   object $payment
	 * @param   string $paymentType
	 * @return  boolean
	 * @since   1.0.0
	 */
	protected function updateVmPlgPaymentType($payment, $paymentType)
	{
		$db = JFactory::getDBO();
		$query = "UPDATE `" . $this->_tablename . "` SET payment_type = '" . $paymentType . "' WHERE id = '" . (int) $payment->id . "'";
		$db->setQuery($query);
		$db->execute();
	}

	/**
	 * This method is fired when showing the order details in the frontend.
	 * It displays the method-specific data.
	 *
	 * @param   string $virtuemartOrderId
	 * @param   string $virtuemartPaymentmethodId
	 * @param   string $paymentName
	 * @return  mixed Null for methods that aren't active, text (HTML) otherwise
	 * @since   1.0.0
	 */
	public function plgVmOnShowOrderFEPayment($virtuemartOrderId, $virtuemartPaymentmethodId, &$paymentName)
	{
		$this->onShowOrderFE($virtuemartOrderId, $virtuemartPaymentmethodId, $paymentName);
	}

	/**
	 * This method is fired when showing the order details in the frontend, for every orderline.
	 * It can be used to display line specific package codes, e.g. with a link to external tracking and
	 * tracing systems
	 *
	 * @param   string $name
	 * @param   string $id
	 * @param   array $data
	 * @return mixed Null for method that aren't active, text (HTML) otherwise
	 * @since   1.0.0
	 */
	public function plgVmDeclarePluginParamsPayment($name, $id, &$data)
	{
		return $this->declarePluginParams('payment', $name, $id, $data);
	}

	/**
	 *
	 * @param   string $name
	 * @param   string $id
	 * @param   object $table
	 * @return boolean
	 * @since   1.0.0
	 */
	public function plgVmSetOnTablePluginParamsPayment($name, $id, &$table)
	{
		return $this->setOnTablePluginParams($name, $id, $table);
	}
}
