<?php
/**
 * @package     VirtueMart.vmpayment
 * @subpackage  vrpayecommerce
 *
 * @copyright Copyright (c) 2004 - 2010 VirtueMart Team. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */

defined('_JEXEC') or die('Restricted access');

if (!class_exists('VersionTracker'))
{
	require JPATH_SITE . DS . 'plugins' . DS . 'vmpayment' . DS . 'vrpayecommerce' . DS . 'vrpayecommerce' . DS . 'helpers' . DS . 'versiontracker.php';
}

/**
 * VRpayecommerceHelper class to get, insert and update vrpayecommerce data etc.
 *
 * @package     VirtueMart.vmpayment
 * @subpackage  vrpayecommerce
 * @since       1.0.0
 */

class VRpayecommerceHelper
{
	/**
	 * plugin client name
	 *
	 * @var    string
	 * @since  1.0.0
	 */
	protected $client = 'CardProcess';

	/**
	 * Shop name
	 *
	 * @var    string
	 * @since  1.0.0
	 */
	protected $shopSystem = 'Virtuemart';

	/**
	 * plugin version
	 *
	 * @var    string
	 * @since  1.0.0
	 */
	protected $version = '1.0.0';

	public function __construct($paymentMethod)
	{
		$this->paymentMethod = $paymentMethod;
		$this->user = JFactory::getUser();
	}

	/**
	 * Get VR pay eCommerce data config
	 *
	 * @param   string  $key
	 * @return  string
	 * @since   1.0.0
	 */
	public function getConfig($key)
	{
		$db = JFactory::getDBO();
		$query = "SELECT payment_params FROM `#__virtuemart_paymentmethods` WHERE  payment_element = 'vrpayecommerce'";
		$db->setQuery($query);
		$params = $db->loadResult();
		$paymentParams = explode("|", $params);

		foreach ($paymentParams as $paymentParam)
		{
			if (empty($paymentParam))
			{
				continue;
			}

			$param = explode('=', $paymentParam);
			$paymentParams[$param[0]] = substr($param[1], 1, -1);
		}

		return $paymentParams[$key];
	}

	/**
	 * Get server mode of payment method
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getServerMode()
	{
		return $this->getConfig($this->paymentMethod . '_server');
	}

	/**
	 * Get test mode of payment method
	 *
	 * @param   string  $key
	 * @return  string
	 * @since   1.0.0
	 */
	public function getTestMode()
	{
		$testMode = "EXTERNAL";

		if ($this->getServerMode() == "LIVE")
		{
			$testMode = false;
		}

		switch ($this->paymentMethod)
		{
			case 'giropay':
				$testMode = "INTERNAL";
				break;
		}

		return $testMode;
	}

	/**
	 * Check availability multichannel at Credit Card (Recurring)
	 *
	 * @return  boolean|string
	 * @since   1.0.0
	 */
	public function isMultiChannel()
	{
		if ($this->paymentMethod == 'cc_saved')
		{
			return $this->getConfig($this->paymentMethod . '_multichannel');
		}

		return false;
	}

	/**
	 * Check availability versiontracker
	 *
	 * @return  boolean
	 * @since   1.1.02
	 */
	public function isVersionTracker()
	{
		return $this->getConfig('general_version_tracker');
	}

	/**
	 * Get minimum age data
	 *
	 * @return  string
	 * @since   1.0.2
	 */
	public function getMinimumAge()
	{
		return $this->getConfig($this->paymentMethod . '_minimum_age');
	}

	/**
	 * Get Partial Capture or Refund
	 *
	 * @return  string
	 * @since   1.0.2
	 */
	public function getPaymentIsPartial()
	{
		return $this->getConfig($this->paymentMethod . '_payment_is_partial');
	}

	/**
	 * Get credential data
	 *
	 * @return  array
	 * @since   1.0.0
	 */
	public function getCredentials()
	{
		$credentials = array(
			'server_mode' => $this->getServerMode(),
			'channel_id'  => $this->getConfig($this->paymentMethod . '_entity_id'),
			'login'       => $this->getConfig('general_login'),
			'password'    => $this->getConfig('general_password')
		);

		if ($this->isMultiChannel())
		{
			$credentials['channel_id_moto'] = $this->getConfig($this->paymentMethod . '_entity_id_moto');
		}

		return $credentials;
	}

	/**
	 * Get payment type selection at Credit Card and Credit Card (Recurring)
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getPaymentTypeSelection()
	{
		return $this->getConfig($this->paymentMethod . '_payment_type');
	}

	/**
	 * Get payment type of all payment method
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getPaymentType()
	{
		switch ($this->paymentMethod)
		{
			case 'cc':
			case 'cc_saved':
			case 'dc':
			case 'dd':
			case 'dd_saved':
			case 'paydirekt':
				$paymentType = $this->getPaymentTypeSelection();
				break;
			case 'klarnainv':
			case 'klarnains':
				$paymentType = 'PA';
				break;
			case 'paypal_saved':
				$paymentType = false;
				break;
			default:
				$paymentType = 'DB';
				break;
		}

		return $paymentType;
	}

	/**
	 * Get the card brand
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getCardBrand()
	{
		$brandCard = $this->getConfig($this->paymentMethod . '_cards');

		if (is_array($brandCard))
		{
			return implode(' ', $brandCard);
		}
		else
		{
			$brandCard = str_replace('"', '', $brandCard);
			$brandCard = str_replace(',', ' ', $brandCard);

			return trim($brandCard);
		}
	}

	/**
	 * Get the brand
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getBrand()
	{
		switch ($this->paymentMethod)
		{
			case 'cc':
			case 'cc_saved':
			case 'dc':
				$brand = $this->getCardBrand();
				break;
			case 'dd':
			case 'dd_saved':
				$brand = 'DIRECTDEBIT_SEPA';
				break;
			case 'klarnainv':
				$brand = 'KLARNA_INVOICE';
				break;
			case 'klarnains':
				$brand = 'KLARNA_INSTALLMENTS';
				break;
			case 'paypal_saved':
				$brand = 'PAYPAL';
				break;
			case 'paydirekt':
				$brand = 'PAYDIREKT';
				break;
			case 'sofort':
				$brand = 'SOFORTUEBERWEISUNG';
				break;
			default:
				$brand = strtoupper($this->paymentMethod);
				break;
		}

		return $brand;
	}

	/**
	 * Get register amount
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getRegisterAmount()
	{
		return $this->getConfig($this->paymentMethod . '_amount');
	}

	/**
	 * Check redirect payment form of payment method
	 *
	 * @return  boolean
	 * @since   1.0.0
	 */
	public function isRedirect()
	{
		switch ($this->paymentMethod)
		{
			case 'paypal':
			case 'paypal_saved':
			case 'paydirekt':
				return true;
			default:
				return false;
		}
	}

	/**
	 * Check availability recurring
	 *
	 * @return  boolean
	 * @since   1.0.0
	 */
	public function isRecurring()
	{
		switch ($this->paymentMethod)
		{
			case 'cc_saved':
			case 'dd_saved':
			case 'paypal_saved':
				return true;
			default:
				return false;
		}
	}

	/**
	 * Get language
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getLang()
	{
		$language = JFactory::getLanguage();
		$tag = strtolower(substr($language->get('tag'), 0, 2));

		return $tag;
	}

	/**
	 * Get IP address
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getCustomerIp()
	{
		if ($_SERVER['REMOTE_ADDR'] == '::1')
		{
			return "127.0.0.1";
		}

		return $_SERVER['REMOTE_ADDR'];
	}

	/**
	 * Get merchant email
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getMerchantEmail()
	{
		if ($this->getConfig('general_merchant_email'))
		{
			$merchantEmail = $this->getConfig('general_merchant_email');
		}
		else
		{
			$db = JFactory::getDBO();
			$db->setQuery("SELECT u.email FROM `#__users` u JOIN `#__user_usergroup_map` g ON u.id = g.user_id WHERE g.group_id = '8'");
			$user = $db->loadAssocList();
			$merchantEmail = $user[0]['email'];
		}

		return $merchantEmail;
	}

	/**
	 * Get version data
	 *
	 * @return  array
	 * @since   1.0.0
	 */
	protected function getVersionData()
	{
		$versionData['transaction_mode'] = $this->getServerMode();
		$versionData['ip_address'] = $_SERVER['SERVER_ADDR'];
		$versionData['shop_version'] = VmConfig::getInstalledVersion();
		$versionData['plugin_version'] = $this->version;
		$versionData['client'] = $this->client;
		$versionData['email'] = $this->getMerchantEmail();
		$versionData['merchant_id'] = $this->getConfig('general_merchant_no');
		$versionData['shop_system'] = $this->shopSystem;
		$versionData['shop_url'] = $this->getConfig('general_shop_url');

		return $versionData;
	}

	/**
	 * Send Version Tracker
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function sendVersionTracker()
	{
		return VersionTracker::sendVersionTracker($this->getVersionData());
	}

	/**
	 * Get recurring group
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getRecurringGroup()
	{
		switch ($this->paymentMethod)
		{
			case 'cc_saved':
				$recurringGroup = 'CC';
				break;
			case 'dd_saved':
				$recurringGroup = 'DD';
				break;
			case 'paypal_saved':
				$recurringGroup = 'VA';
				break;
			default:
				$recurringGroup = '';
				break;
		}

		return $recurringGroup;
	}

	/**
	 * Get registered payment data
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	public function getRegistrations()
	{
		$db = JFactory::getDBO();
		$credentials = $this->getCredentials();
		$query = "SELECT * FROM `#__virtuemart_payment_plg_vrpayecommerce_recurring`
			WHERE cust_id = '" . (int) $this->user->id . "'
			AND server_mode = '" . $credentials['server_mode'] . "'
			AND channel_id = '" . $credentials['channel_id'] . "'
			AND payment_group = '" . $this->getRecurringGroup() . "'";
		$db->setQuery($query);

		return $db->loadAssocList();
	}

	/**
	 * Check availability registered payment data by ref_id
	 *
	 * @param   string $registrationId
	 * @return  string
	 * @since   1.0.0
	 */
	public function isRegistered($registrationId)
	{
		$db = JFactory::getDBO();
		$query = "SELECT id FROM `#__virtuemart_payment_plg_vrpayecommerce_recurring`
			WHERE ref_id = '" . $db->escape($registrationId) . "'";
		$db->setQuery($query);

		if ($db->loadResult())
		{
			return true;
		}

		return false;
	}

	/**
	 * Check availability default of payment method recurring
	 *
	 * @return  integer
	 * @since   1.0.0
	 */
	public function checkDefault()
	{
		$db = JFactory::getDBO();
		$query = "SELECT id FROM `#__virtuemart_payment_plg_vrpayecommerce_recurring`
			WHERE cust_id = '" . (int) $this->user->id . "' AND
			payment_group = '" . $this->getRecurringGroup() . "' AND payment_default = '1'";
		$db->setQuery($query);

		if (!$db->loadResult())
		{
			return 1;
		}

		return 0;
	}

	/**
	 * Get account data of payment recurring
	 *
	 * @param   array $paymentStatus
	 * @return  array
	 * @since   1.0.0
	 */
	public function getAccount($paymentStatus)
	{
		switch ($this->paymentMethod)
		{
			case 'cc_saved':
				$account = $paymentStatus['card'];
				$account['email'] = '';
				break;
			case 'dd_saved':
				$account = $paymentStatus['bankAccount'];
				$account['last4Digits'] = substr($account['iban'], -4);
				$account['email'] = '';
				$account['expiryMonth'] = '';
				$account['expiryYear'] = '';
				break;
			case 'paypal_saved':
				$account = $paymentStatus['virtualAccount'];
				$account['email'] = $account['accountId'];
				$account['last4Digits'] = '';
				$account['expiryMonth'] = '';
				$account['expiryYear'] = '';
				break;
		}

		return $account;
	}

	/**
	 * Insert new registered payment data to the database
	 *
	 * @param   string $registrationId
	 * @param   array $paymentStatus
	 * @param   boolean $check
	 * @return  void
	 * @since   1.0.0
	 */
	public function insertPaymentAccount($registrationId, $paymentStatus, $check=false)
	{
		$isRegistered = false;

		if ($check)
		{
			$isRegistered = $this->isRegistered($registrationId);
		}

		if (!$isRegistered)
		{
			$db = JFactory::getDBO();
			$default = $this->checkDefault();
			$account = $this->getAccount($paymentStatus);
			$credentials = $this->getCredentials();
			$query = "INSERT INTO `#__virtuemart_payment_plg_vrpayecommerce_recurring` (
				cust_id,
				payment_group,
				brand,
				holder,
				email,
				last4digits,
				expiry_month,
				expiry_year,
				server_mode,
				channel_id,
				ref_id,
				payment_default
				) VALUES (
				'" . (int) $this->user->id . "',
				'" . $this->getRecurringGroup() . "',
				'" . $paymentStatus['paymentBrand'] . "',
				'" . $account['holder'] . "',
				'" . $account['email'] . "',
				'" . $account['last4Digits'] . "',
				'" . $account['expiryMonth'] . "',
				'" . $account['expiryYear'] . "',
				'" . $credentials['server_mode'] . "',
				'" . $credentials['channel_id'] . "',
				'" . $db->escape($registrationId) . "',
				'" . $default . "')";
			$db->setQuery($query);
			$db->execute();
		}
	}

	/**
	 * Update new registered payment data to the database
	 *
	 * @param   string $recurringId
	 * @param   string $registrationId
	 * @param   array $paymentStatus
	 * @return  void
	 * @since   1.0.0
	 */
	public function updatePaymentAccount($recurringId, $registrationId, $paymentStatus)
	{
		$db = JFactory::getDBO();
		$account = $this->getAccount($paymentStatus);
		$credentials = $this->getCredentials();
		$query = "UPDATE `#__virtuemart_payment_plg_vrpayecommerce_recurring`
			SET cust_id = '" . (int) $this->user->id . "',
			payment_group = '" . $this->getRecurringGroup() . "',
			brand = '" . $paymentStatus['paymentBrand'] . "',
			email = '" . $account['email'] . "',
			holder = '" . $account['holder'] . "',
			last4digits = '" . $account['last4Digits'] . "',
			expiry_month = '" . $account['expiryMonth'] . "',
			expiry_year = '" . $account['expiryYear'] . "',
			server_mode = '" . $credentials['server_mode'] . "',
			channel_id = '" . $credentials['channel_id'] . "',
			ref_id = '" . $db->escape($registrationId) . "' where id = '" . (int) $recurringId . "'";
		$db->setQuery($query);
		$db->execute();
	}

	/**
	 * Determine to insert or update payment data to the database
	 *
	 * @param   string $recurringId
	 * @param   string $registrationId
	 * @param   array $paymentStatus
	 * @return  void
	 * @since   1.0.0
	 */
	public function insertOrUpdateRegistration($recurringId, $registrationId, $paymentStatus)
	{
		if ($recurringId)
		{
			$this->updatePaymentAccount($recurringId, $registrationId, $paymentStatus);
		}
		else
		{
			$this->insertPaymentAccount($registrationId, $paymentStatus);
		}
	}
}
