<?php
/**
 * @package     VirtueMart.vmpayment
 * @subpackage  vrpayecommerce
 *
 * @copyright Copyright (c) 2004 - 2010 VirtueMart Team. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * VirtueMart is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */

defined('_JEXEC') or die('Restricted access');

/**
 * VersionTracker class to create payment widget, send payment request and receive payment response from payment gateway
 *
 * @package     VirtueMart.vmpayment
 * @subpackage  vrpayecommerce
 * @since       1.0.0
 */
class VersionTracker
{
	/**
	 * version tracker URL
	 *
	 * @var    string
	 * @since  1.0.0
	 */
	protected static $versionTrackerUrl = 'http://api.dbserver.payreto.eu/v1/tracker';

	/**
	 * Get version tracker URL
	 *
	 * @return  string
	 * @since   1.0.0
	 */
	private static function getVersionTrackerUrl()
	{
		return self::$versionTrackerUrl;
	}

	/**
	 * Get response data from version tracker gateway
	 *
	 * @param   array  $data
	 * @param   string  $url
	 * @return  boolean|string
	 * @since   1.0.0
	 */
	private static function getResponseData($data, $url)
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

		$response = curl_exec($ch);

		if (curl_errno($ch))
		{
			return false;
		}

		curl_close($ch);

		return json_decode($response, true);
	}

	/**
	 * Get version tracker parameter
	 *
	 * @param   array  $versionData
	 * @return  string
	 * @since   1.0.0
	 */
	private static function getVersionTrackerParameter($versionData)
	{
		$data = 'transaction_mode=' . $versionData['transaction_mode'] .
				'&ip_address=' . $versionData['ip_address'] .
				'&shop_version=' . $versionData['shop_version'] .
				'&plugin_version=' . $versionData['plugin_version'] .
				'&client=' . $versionData['client'] .
				'&hash=' . md5($versionData['shop_version'] . $versionData['plugin_version'] . $versionData['client']);

		if ($versionData['shop_system'])
		{
			$data .= '&shop_system=' . $versionData['shop_system'];
		}

		if ($versionData['email'])
		{
			$data .= '&email=' . $versionData['email'];
		}

		if ($versionData['merchant_id'])
		{
			$data .= '&merchant_id=' . $versionData['merchant_id'];
		}

		if ($versionData['shop_url'])
		{
			$data .= '&shop_url=' . $versionData['shop_url'];
		}

		return $data;
	}

	/**
	 * Send version tracker to version tracker gateway
	 *
	 * @param   array  $versionData
	 * @return  string
	 * @since   1.0.0
	 */
	public static function sendVersionTracker($versionData)
	{
		$postData = self::getVersionTrackerParameter($versionData);
		$url = self::getVersionTrackerUrl();

		return self::getResponseData($postData, $url);
	}
}
